@@ -81,7 +81,6 @@ static void initialise_toolbar(void);
 #endif
 
 
-
 #ifdef FEAT_MENU
 /*
  * Figure out how high the menu bar is at the moment.
@@ -149,6 +148,7 @@ _OnScroll(
     int		nPos;
 #endif
     static UINT	prev_code = 0;   /* code of previous call */
+    int		dont_scroll_save = dont_scroll;
 
     sb = gui_mswin_find_scrollbar(hwndCtl);
     if (sb == NULL)
@@ -239,8 +239,14 @@ _OnScroll(
 
     /* Don't let us be interrupted here by another message. */
     s_busy_processing = TRUE;
+
+    /* When "allow_scrollbar" is FALSE still need to remember the new
+     * position, but don't actually scroll by setting "dont_scroll". */
+    dont_scroll = !allow_scrollbar;
+
     gui_drag_scrollbar(sb, val, dragging);
     s_busy_processing = FALSE;
+    dont_scroll = dont_scroll_save;
 
     return 0;
 }
@@ -350,10 +356,6 @@ _WndProc(
 #endif
 	    return 0;
 
-    case WM_CREATE:	/* HANDLE_MSG doesn't seem to handle this one */
-	return _OnCreate (hwnd, (LPCREATESTRUCT)lParam);
-
-
 #if defined(MENUHINTS) && defined(FEAT_MENU)
     case WM_MENUSELECT:
 	if (((UINT) LOWORD(lParam)
@@ -460,7 +462,7 @@ gui_mch_init(void)
 #endif
     gui.border_width = 0;
 
-    gui.currBgColor = (long) -1;
+    gui.currBgColor = INVALCOLOR;
 
     s_brush = CreateSolidBrush(GetSysColor(COLOR_BTNFACE));
 
@@ -758,9 +760,6 @@ gui_mch_draw_string(
      */
     HBRUSH	hbr;
     RECT	rc;
-#ifdef FEAT_MBYTE_IME
-    char	*szComp;
-#endif
 
     if (!(flags & DRAW_TRANSP))
     {
