@@ -151,7 +151,7 @@ mac_expandpath(
 
     /* compile the regexp into a program */
     regmatch.rm_ic = FALSE;			/* Don't ever ignore case */
-    regmatch.regprog = vim_regcomp(pat, TRUE);
+    regmatch.regprog = vim_regcomp(pat, RE_MAGIC);
     vim_free(pat);
 
     if (regmatch.regprog == NULL)
@@ -390,7 +390,7 @@ unix_expandpath(gap, path, wildoff, flags)
 #else
     regmatch.rm_ic = FALSE;		/* Don't ever ignore case */
 #endif
-    regmatch.regprog = vim_regcomp(pat, TRUE);
+    regmatch.regprog = vim_regcomp(pat, RE_MAGIC);
     vim_free(pat);
 
     if (regmatch.regprog == NULL)
@@ -694,22 +694,24 @@ mch_get_user_name(s, len)
 }
 
 /*
- * Insert host name is s[len].
+ * Copy host name into s[len].
  */
-
     void
 mch_get_host_name(s, len)
     char_u	*s;
     int		len;
 {
 #if defined(__MRC__) || defined(__SC__) || defined(__APPLE_CC__)
-    s[0] = '\0'; /* TODO: use Gestalt information */
+    STRNCPY(s, "Mac", len); /* TODO: use Gestalt information */
 #else
     struct utsname vutsname;
 
-    uname(&vutsname);
-    STRNCPY(s, vutsname.nodename, len);
+    if (uname(&vutsname) < 0)
+	*s = NUL;
+    else
+	STRNCPY(s, vutsname.nodename, len);
 #endif
+    s[len - 1] = NUL;	/* make sure it's terminated */
 }
 
 /*
@@ -1081,14 +1083,20 @@ mch_isdir(name)
      */
     struct stat statb;
 
+#if defined(TARGET_API_MAC_CARBON) && TARGET_API_MAC_CARBON
     /* For some reason the name is sometimes empty,
      * (such as for a not yet named file). An empty
      * filename is interpreted by the MacOS version
      * of stat (at least under Codewarrior) as the
      * current directory.
      */
+    /* AK 20020413
+     * This is required for Carbon but breaks the
+     * explorer plugin in Classic
+     */
     if (name[0] == NULL)
 	return FALSE;
+#endif
 
     if (stat((char *)name, &statb))
 	return FALSE;
