@@ -139,8 +139,11 @@ mygotoxy(int x, int y)
     static void
 set_sys_cursor(void)
 {
-    myflush();
-    gotoxy(S_iCurrentColumn + 1, S_iCurrentRow + 1);
+    if (term_console && full_screen)
+    {
+	myflush();
+	gotoxy(S_iCurrentColumn + 1, S_iCurrentRow + 1);
+    }
 }
 
     static void
@@ -1515,6 +1518,7 @@ mch_get_host_name(
 #else
     STRNCPY(s, "PC (16 bits Vim)", len);
 #endif
+    s[len - 1] = NUL;	/* make sure it's terminated */
 }
 
 /*
@@ -2103,6 +2107,7 @@ mch_call_shell(
     int		options)
 {
     int		x;
+    int		tmode = cur_tmode;
 #ifndef DJGPP
     char_u	*newcmd;
 #endif
@@ -2152,7 +2157,8 @@ mch_call_shell(
     signal(SIGQUIT, SIG_DFL);
     signal(SIGTERM, SIG_DFL);
 #endif
-    settmode(TMODE_RAW);	/* set to raw mode */
+    if (tmode == TMODE_RAW)
+	settmode(TMODE_RAW);	/* set to raw mode */
     set_interrupts(TRUE);	/* catch interrupts */
 
     if (x && !(options & SHELL_SILENT) && !emsg_silent)
@@ -2236,7 +2242,7 @@ mch_chdir(char *path)
 	return 0;
     if (path[1] == ':')		    /* has a drive name */
     {
-	if (change_drive(TO_LOWER(path[0]) - 'a' + 1))
+	if (change_drive(TOLOWER_ASC(path[0]) - 'a' + 1))
 	    return -1;		    /* invalid drive name */
 	path += 2;
     }
