@@ -297,19 +297,19 @@ dyn_libintl_end()
     dyn_libintl_bindtextdomain	= null_libintl_bindtextdomain;
 }
 
-    char*
+    static char *
 null_libintl_gettext(const char* msgid)
 {
     return (char*)msgid;
 }
 
-    char*
+    static char *
 null_libintl_bindtextdomain(const char* domainname, const char* dirname)
 {
     return NULL;
 }
 
-    char*
+    static char *
 null_libintl_textdomain(const char* domainname)
 {
     return NULL;
@@ -786,7 +786,7 @@ decode_mouse_event(
     static s_dwLastClickTime = 0;
     static BOOL s_fNextIsMiddle = FALSE;
 
-    static int cButtons = 0;	/* number of buttons supported */
+    static DWORD cButtons = 0;	/* number of buttons supported */
 
     const DWORD LEFT = FROM_LEFT_1ST_BUTTON_PRESSED;
     const DWORD MIDDLE = FROM_LEFT_2ND_BUTTON_PRESSED;
@@ -1507,8 +1507,10 @@ mch_init()
     /*
      * Vim's own clipboard format recognises whether the text is char, line, or
      * rectangular block.  Only useful for copying between two Vims.
+     * "VimClipboard" was used for previous versions, using the first
+     * character to specify MCHAR, MLINE or MBLOCK.
      */
-    clip_star.format = RegisterClipboardFormat("VimClipboard");
+    clip_star.format = RegisterClipboardFormat("VimClipboard2");
 #endif
 }
 
@@ -2002,8 +2004,10 @@ mch_init()
     /*
      * Vim's own clipboard format recognises whether the text is char, line, or
      * rectangular block.  Only useful for copying between two Vims.
+     * "VimClipboard" was used for previous versions, using the first
+     * character to specify MCHAR, MLINE or MBLOCK.
      */
-    clip_star.format = RegisterClipboardFormat("VimClipboard");
+    clip_star.format = RegisterClipboardFormat("VimClipboard2");
 #endif
 
     /* This will be NULL on anything but NT 4.0 */
@@ -2118,6 +2122,7 @@ fname_case(
 	/* copy leading drive letter */
 	*ptrue++ = *porig++;
 	*ptrue++ = *porig++;
+	*ptrue = NUL;	    /* in case nothing follows */
     }
 
     while (*porig != NUL)
@@ -2223,16 +2228,13 @@ mch_get_host_name(
     char_u	*s,
     int		len)
 {
-    char szHostName[MAX_COMPUTERNAME_LENGTH + 1];
-    DWORD cch = sizeof szHostName;
+    DWORD cch = len;
 
-    if (GetComputerName(szHostName, &cch))
+    if (!GetComputerName(s, &cch))
     {
-	STRCPY(s, "PC ");
-	STRNCPY(s + 3, szHostName, len - 3);
-    }
-    else
 	STRNCPY(s, "PC (Win32 Vim)", len);
+	s[len - 1] = NUL;	/* make sure it's terminated */
+    }
 }
 
 
@@ -2322,13 +2324,16 @@ mch_isdir(char_u *name)
 
 /*
  * Return TRUE if file or directory "name" is writable (not readonly).
+ * Strange semantics of Win32: a readonly directory is writable, but you can't
+ * delete a file.  Let's say this means it is writable.
  */
     int
 mch_writable(char_u *name)
 {
     int perm = mch_getperm(name);
 
-    return (perm != -1 && !(perm & FILE_ATTRIBUTE_READONLY));
+    return (perm != -1 && (!(perm & FILE_ATTRIBUTE_READONLY)
+				       || (perm & FILE_ATTRIBUTE_DIRECTORY)));
 }
 
 #if defined(FEAT_EVAL) || defined(PROTO)
@@ -2803,6 +2808,8 @@ mch_system(char *cmd, int options)
     if (g_PlatformId != VER_PLATFORM_WIN32s)
     {
 #ifdef FEAT_GUI
+	int	    delay = 1;
+
 	/* Keep updating the window while waiting for the shell to finish. */
 	for (;;)
 	{
@@ -2813,8 +2820,14 @@ mch_system(char *cmd, int options)
 		TranslateMessage(&msg);
 		DispatchMessage(&msg);
 	    }
-	    if (WaitForSingleObject(pi.hProcess, 100) != WAIT_TIMEOUT)
+	    if (WaitForSingleObject(pi.hProcess, delay) != WAIT_TIMEOUT)
 		break;
+
+	    /* We start waiting for a very short time and then increase it, so
+	     * that we respond quickly when the process is quick, and don't
+	     * consume too much overhead when it's slow. */
+	    if (delay < 50)
+		delay += 10;
 	}
 #else
 	WaitForSingleObject(pi.hProcess, INFINITE);
@@ -2846,7 +2859,7 @@ mch_system(char *cmd, int options)
     CloseHandle(pi.hProcess);
 
     /* Try to get input focus back.  Doesn't always work though. */
-    SetFocus(hwnd);
+    PostMessage(hwnd, WM_SETFOCUS, 0, 0);
 
     return ret;
 }
@@ -2864,7 +2877,8 @@ mch_call_shell(
     char_u *cmd,
     int options)	/* SHELL_*, see vim.h */
 {
-    int	    x = 0;
+    int		x = 0;
+    int		tmode = cur_tmode;
 #ifdef FEAT_TITLE
     char szShellTitle[512];
 
@@ -3025,7 +3039,8 @@ mch_call_shell(
 	}
     }
 
-    settmode(TMODE_RAW);	    /* set to raw mode */
+    if (tmode == TMODE_RAW)
+	settmode(TMODE_RAW);	/* set to raw mode */
 
     /* Print the return value, unless "vimrun" was used. */
     if (x != 0 && !(options & SHELL_SILENT) && !emsg_silent
@@ -4009,13 +4024,169 @@ default_shell()
 }
 
 /*
- * mch_access() was used to support ACLs under Windows NT/2K/XP(?).
- * Unfortunately the ACL system functions are buggy, we couldn't make it work,
- * removed for now.
+ * mch_access() extends access() to do more detailed check on network drives.
  * Returns 0 if file "n" has access rights according to "p", -1 otherwise.
  */
     int
 mch_access(char *n, int p)
 {
-    return access(n, p);
+    HANDLE  hFile;
+    DWORD am;
+
+    if (mch_isdir(n))
+    {
+	char TempName[_MAX_PATH + 16] = "";
+
+	if (p & R_OK)
+	{
+	    /* Read check is performed by seeing if we can do a find file on
+	     * the directory for any file
+	     */
+	    char *pch;
+	    WIN32_FIND_DATA d;
+
+	    STRNCPY(TempName, n, _MAX_PATH);
+	    pch = TempName + STRLEN(TempName) - 1;
+	    if (*pch != '\\' && *pch != '/')
+		*pch++ = '\\';
+	    *pch++ = '*';
+	    *pch = NUL;
+
+	    hFile = FindFirstFile(TempName, &d);
+	    if (hFile == INVALID_HANDLE_VALUE)
+		return -1;
+	    (void)FindClose(hFile);
+	}
+
+	if (p & W_OK)
+	{
+	    /* Trying to create a temporary file in the directory should catch
+	     * directories on read-only network shares.  However, in
+	     * directories whose ACL allows writes but denies deletes will end
+	     * up keeping the temporary file :-(
+	     */
+	    if (!GetTempFileName(n, "VIM", 0, TempName))
+		return -1;
+	    mch_remove((char_u *)TempName);
+	}
+    }
+    else
+    {
+	/* Trying to open the file for the required access does ACL, read-only
+	 * network share, and file attribute checks.
+	 */
+	am = ((p & W_OK) ? GENERIC_WRITE : 0)
+		| ((p & R_OK) ? GENERIC_READ : 0);
+	hFile = CreateFile(n, am, 0, NULL, OPEN_EXISTING, 0, NULL);
+	if (hFile == INVALID_HANDLE_VALUE)
+	    return -1;
+	CloseHandle(hFile);
+    }
+    return 0;
+}
+
+/*
+ * SUB STREAM:
+ *
+ * NTFS can have sub streams for each file.  Normal contents of file is
+ * stored in main stream, and external contents (author information and
+ * title and so on) can be stored in sub stream.  After Windows 2000, user
+ * can access and store those informations in sub streams via explorer's
+ * property menuitem in right click menu.  Those informations in sub streams
+ * were lost when copy only main streams.  So we have to copy sub streams.
+ *
+ * http://msdn.microsoft.com/library/en-us/dnw2k/html/ntfs5.asp
+ */
+
+    static char *
+get_infostream_name(char *name, char *substream)
+{
+    int len;
+    char *newname;
+
+    len = strlen(name) + strlen(substream) + 2;
+    newname = malloc(len);
+    strcpy(newname, name);
+    strcat(newname, ":");
+    strcat(newname, substream);
+    return newname;
+}
+
+    static int
+copy_substream(char *from, char *to, char *substream)
+{
+    int	    retval = 0;
+    HANDLE  hFrom, hTo;
+    char    *from_info, *to_info;
+
+    from_info	= get_infostream_name(from, substream);
+    to_info	= get_infostream_name(to, substream);
+    hFrom = CreateFile(from_info, GENERIC_READ, 0, NULL, OPEN_EXISTING,
+	    FILE_ATTRIBUTE_NORMAL, NULL);
+    hTo = CreateFile(to_info, GENERIC_WRITE, 0, NULL, OPEN_ALWAYS,
+	    FILE_ATTRIBUTE_NORMAL, NULL);
+
+    if (hFrom != INVALID_HANDLE_VALUE && hTo != INVALID_HANDLE_VALUE)
+    {
+	/* Just copy information */
+	DWORD dwCopy, dwSize, dwRead = 0, dwWrite = 0;
+	char buf[4096];
+
+	retval = 1;
+	dwCopy = GetFileSize(hFrom, NULL);
+	while (dwCopy > 0)
+	{
+	    dwSize = dwCopy > sizeof(buf) ? sizeof(buf) : dwCopy;
+	    if (!ReadFile(hFrom, buf, dwSize, &dwRead, NULL)
+		    || dwRead != dwSize
+		    || !WriteFile(hTo, buf, dwSize, &dwWrite, NULL)
+		    || dwWrite != dwSize)
+	    {
+		retval = 0;
+		break;
+	    }
+	    dwCopy -= dwSize;
+	}
+    }
+    else if (hFrom == INVALID_HANDLE_VALUE && hTo != INVALID_HANDLE_VALUE)
+    {
+	/* No source information, delete destination */
+	CloseHandle(hTo);
+	hTo = INVALID_HANDLE_VALUE;
+	retval = DeleteFile(to_info);
+	retval = 1;
+    }
+    else if (hFrom == INVALID_HANDLE_VALUE && hTo == INVALID_HANDLE_VALUE)
+    {
+	retval = 1;
+    }
+
+    if (hFrom != INVALID_HANDLE_VALUE)
+	CloseHandle(hFrom);
+    if (hTo != INVALID_HANDLE_VALUE)
+	CloseHandle(hTo);
+    free(from_info);
+    free(to_info);
+
+    return 1;
+}
+
+    static int
+copy_infostreams(char_u *from, char_u *to)
+{
+    if (!copy_substream(from, to, "\05SummaryInformation"))
+	goto FAILTOCOPY;
+    if (!copy_substream(from, to, "\05DocumentSummaryInformation"))
+	goto FAILTOCOPY;
+    if (!copy_substream(from, to, "\05SebiesnrMkudrfcoIaamtykdDa"))
+	goto FAILTOCOPY;
+    return 1;
+FAILTOCOPY:
+    return 0;
+}
+
+    int
+mch_copy_file_attribute(char_u *from, char_u *to)
+{
+    return copy_infostreams(from, to);
 }
